@@ -20,21 +20,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/**
- * SECTION:pango-renderer
- * @short_description:Rendering driver base class
- * @title:PangoRenderer
- *
- * #PangoRenderer is a base class that contains the necessary logic for
- * rendering a #PangoLayout or #PangoLayoutLine. By subclassing
- * #PangoRenderer and overriding operations such as @draw_glyphs and
- * @draw_rectangle, renderers for particular font backends and
- * destinations can be created.
- */
 #include "config.h"
 #include <math.h>
 
-#include "pango-font-private.h"
 #include "pangoft2-private.h"
 
 /* for compatibility with older freetype versions */
@@ -148,7 +136,7 @@ pango_ft2_font_render_box_glyph (int      width,
       offset1 = box->bitmap.pitch * (MIN (1 + j, height - 1));
       offset2 = box->bitmap.pitch * (MAX (box->bitmap.rows - 2 - j, 0));
       for (i = 1;
-	   i < (int) box->bitmap.width - 1;
+	   i < box->bitmap.width - 1;
 	   i++)
 	{
 	  box->bitmap.buffer[offset1 + i] = 0xff;
@@ -158,9 +146,9 @@ pango_ft2_font_render_box_glyph (int      width,
   for (j = 0; j < line_width; j++)
     {
       offset1 = MIN (1 + j, width - 1);
-      offset2 = MAX ((int) box->bitmap.width - 2 - j, 0);
+      offset2 = MAX (box->bitmap.width - 2 - j, 0);
       for (i = box->bitmap.pitch;
-	   i < (int) (box->bitmap.rows - 1) * box->bitmap.pitch;
+	   i < (box->bitmap.rows - 1) * box->bitmap.pitch;
 	   i += box->bitmap.pitch)
 	{
 	  box->bitmap.buffer[offset1 + i] = 0xff;
@@ -175,7 +163,7 @@ pango_ft2_font_render_box_glyph (int      width,
       offset1 = PANGO_SCALE;
       offset2 = PANGO_SCALE * MAX (width - line_width - 1, 0) ;
       for (i = box->bitmap.pitch;
-	   i < (int) (box->bitmap.rows - 1) * box->bitmap.pitch;
+	   i < (box->bitmap.rows - 1) * box->bitmap.pitch;
 	   i += box->bitmap.pitch)
         {
 	  for (j = 0; j < line_width; j++)
@@ -222,7 +210,7 @@ pango_ft2_font_render_glyph (PangoFont *font,
       return box;
     }
 
-  face = pango_ft2_font_get_face (font);
+  face = pango_fc_font_lock_face (PANGO_FC_FONT (font));
 
   if (face)
     {
@@ -243,6 +231,8 @@ pango_ft2_font_render_glyph (PangoFont *font,
       rendered->bitmap_left = face->glyph->bitmap_left;
       rendered->bitmap_top = face->glyph->bitmap_top;
 
+      pango_fc_font_unlock_face (PANGO_FC_FONT (font));
+
       if (G_UNLIKELY (!rendered->bitmap.buffer)) {
         g_slice_free (PangoFT2RenderedGlyph, rendered);
 	return NULL;
@@ -252,6 +242,8 @@ pango_ft2_font_render_glyph (PangoFont *font,
     }
   else
     {
+      pango_fc_font_unlock_face (PANGO_FC_FONT (font));
+
 generic_box:
       return  pango_ft2_font_render_box_glyph (PANGO_UNKNOWN_GLYPH_WIDTH,
 					       PANGO_UNKNOWN_GLYPH_HEIGHT,
@@ -304,12 +296,12 @@ pango_ft2_renderer_draw_glyph (PangoRenderer *renderer,
     }
 
   x_start = MAX (0, - (ixoff + rendered_glyph->bitmap_left));
-  x_limit = MIN ((int) rendered_glyph->bitmap.width,
-		 (int) (bitmap->width - (ixoff + rendered_glyph->bitmap_left)));
+  x_limit = MIN (rendered_glyph->bitmap.width,
+		 bitmap->width - (ixoff + rendered_glyph->bitmap_left));
 
   y_start = MAX (0,  - (iyoff - rendered_glyph->bitmap_top));
-  y_limit = MIN ((int) rendered_glyph->bitmap.rows,
-		 (int) (bitmap->rows - (iyoff - rendered_glyph->bitmap_top)));
+  y_limit = MIN (rendered_glyph->bitmap.rows,
+		 bitmap->rows - (iyoff - rendered_glyph->bitmap_top));
 
   src = rendered_glyph->bitmap.buffer +
     y_start * rendered_glyph->bitmap.pitch;
@@ -335,7 +327,6 @@ pango_ft2_renderer_draw_glyph (PangoRenderer *renderer,
 		  break;
 		case 0xff:
 		  *d = 0xff;
-                  break;
 		default:
 		  *d = MIN ((gushort) *d + (gushort) *s, 0xff);
 		  break;
@@ -405,7 +396,7 @@ draw_simple_trap (PangoRenderer *renderer,
   double dy = b->y - t->y;
   guchar *dest;
 
-  if (iy < 0 || iy >= (int) bitmap->rows)
+  if (iy < 0 || iy >= bitmap->rows)
     return;
   dest = bitmap->buffer + iy * bitmap->pitch;
 
@@ -419,8 +410,8 @@ draw_simple_trap (PangoRenderer *renderer,
   else
     x2 = ceil (b->x2);
 
-  x1 = CLAMP (x1, 0, (int) bitmap->width);
-  x2 = CLAMP (x2, 0, (int) bitmap->width);
+  x1 = CLAMP (x1, 0, bitmap->width);
+  x2 = CLAMP (x2, 0, bitmap->width);
 
   for (x = x1; x < x2; x++)
     {
@@ -701,8 +692,7 @@ pango_ft2_render_layout_line (FT_Bitmap       *bitmap,
  * pango_ft2_render_transformed:
  * @bitmap:  the FreeType2 bitmap onto which to draw the string
  * @font:    the font in which to draw the string
- * @matrix:  (nullable): a #PangoMatrix, or %NULL to use an identity
- *           transformation
+ * @matrix:  a #PangoMatrix, or %NULL to use an identity transformation
  * @glyphs:  the glyph string to draw
  * @x:       the x position of the start of the string (in Pango
  *           units in user space coordinates)
